/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import { getColumnLabel, getNumberFormatter, getTimeFormatter, } from '@superset-ui/core';
import { formatSeriesName } from './series';
const getCrossFilterDataMask = (selectedValues, groupby, labelMap) => (value) => {
    const selected = Object.values(selectedValues);
    let values;
    if (selected.includes(value)) {
        values = selected.filter(v => v !== value);
    }
    else {
        values = [value];
    }
    const groupbyValues = values.map(value => labelMap[value]);
    return {
        dataMask: {
            extraFormData: {
                filters: values.length === 0
                    ? []
                    : groupby.map((col, idx) => {
                        const val = groupbyValues.map(v => v[idx]);
                        if (val === null || val === undefined)
                            return {
                                col,
                                op: 'IS NULL',
                            };
                        return {
                            col,
                            op: 'IN',
                            val: val,
                        };
                    }),
            },
            filterState: {
                value: groupbyValues.length ? groupbyValues : null,
                selectedValues: values.length ? values : null,
            },
        },
        isCurrentValueSelected: selected.includes(value),
    };
};
export const clickEventHandler = (getCrossFilterDataMask, setDataMask, emitCrossFilters) => ({ name }) => {
    if (!emitCrossFilters) {
        return;
    }
    const dataMask = getCrossFilterDataMask(name)?.dataMask;
    if (dataMask) {
        setDataMask(dataMask);
    }
};
export const contextMenuEventHandler = (groupby, onContextMenu, labelMap, getCrossFilterDataMask, formData, coltypeMapping) => (e) => {
    if (onContextMenu) {
        e.event.stop();
        const pointerEvent = e.event.event;
        const drillFilters = [];
        if (groupby.length > 0) {
            const values = labelMap[e.name];
            groupby.forEach((dimension, i) => {
                drillFilters.push({
                    col: dimension,
                    op: '==',
                    val: values[i],
                    formattedVal: formatSeriesName(values[i], {
                        timeFormatter: getTimeFormatter(formData.dateFormat),
                        numberFormatter: getNumberFormatter(formData.numberFormat),
                        coltype: coltypeMapping?.[getColumnLabel(dimension)],
                    }),
                });
            });
        }
        onContextMenu(pointerEvent.clientX, pointerEvent.clientY, {
            drillToDetail: drillFilters,
            crossFilter: getCrossFilterDataMask(e.name),
            drillBy: { filters: drillFilters, groupbyFieldName: 'groupby' },
        });
    }
};
export const allEventHandlers = (transformedProps) => {
    const { groupby, onContextMenu, setDataMask, labelMap, emitCrossFilters, selectedValues, coltypeMapping, formData, } = transformedProps;
    const eventHandlers = {
        click: clickEventHandler(getCrossFilterDataMask(selectedValues, groupby, labelMap), setDataMask, emitCrossFilters),
        contextmenu: contextMenuEventHandler(groupby, onContextMenu, labelMap, getCrossFilterDataMask(selectedValues, groupby, labelMap), formData, coltypeMapping),
    };
    return eventHandlers;
};
//# sourceMappingURL=eventHandlers.js.map